import os
import re
from datetime import datetime
from os.path import basename
from pathlib import Path

from PIL import Image
from PIL.ExifTags import TAGS
from PySide6 import QtWidgets
from PySide6.QtGui import QCursor, Qt, QFont
from PySide6.QtWidgets import QLabel, QVBoxLayout, QListWidget, QHBoxLayout, QPushButton, QDialog, \
    QSizePolicy, QFileDialog
from natsort import natsorted

from imgmanip.dialogs.sort_images_dialog import SortImagesDialog
from imgmanip.functions.file_size import file_size
from imgmanip.models.sort import Sort


class LeftPart(QVBoxLayout):
    def __init__(self, root_widget):
        # Right part
        super().__init__()

        self.root_widget = root_widget
        # Default sort_type
        self.sort_type = Sort.UNSORTED

        self.list_of_images = []

        self.default_list_font = QFont()
        self.default_list_font.setPixelSize(13)

        smaller_font = QFont()
        smaller_font.setPixelSize(12)

        self.setObjectName(u"left_vbox")

        # "Images" text
        self.images_label = QLabel()
        self.images_label.setObjectName(u"images_label")
        self.images_label.setText('Images')
        # List of the images
        self.images_list = QListWidget()
        self.images_list.setObjectName(u"images_list")
        self.images_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.images_list.setStyleSheet(
            'QListView::item:selected{background-color: #99A2FF; color: #111111; font-weight: bold}')
        # self.images_list.itemSelectionChanged.connect(self.root_widget.preview_image)
        self.images_list.setFont(self.default_list_font)
        self.images_list.setMaximumWidth(300)
        self.images_list.installEventFilter(self)
        # Images count
        self.images_count_label = QLabel()
        self.images_count_label.setFont(smaller_font)
        self.images_count_label.setText(f'Images count: {len(self.list_of_images)}')
        # Properties of the image
        self.properties_list = QListWidget()
        self.properties_list.setObjectName(u"properties_list")
        self.properties_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.properties_list.setStyleSheet(
            'QListView::item:selected{background-color: #99A2FF; color: #111111; font-weight: bold;}')
        self.properties_list.setFont(self.default_list_font)
        self.properties_list.setMaximumWidth(300)
        self.properties_list.installEventFilter(self)

        size_policy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        size_policy.setHorizontalStretch(0)
        size_policy.setVerticalStretch(0)
        size_policy.setHeightForWidth(self.images_list.sizePolicy().hasHeightForWidth())
        self.images_list.setSizePolicy(size_policy)

        # Horizontal layout with add and remove button (IMAGES)
        self.images_buttons_h_box = QHBoxLayout()
        self.images_buttons_h_box.setObjectName(u"images_buttons_h_box")
        # add button
        self.add_images_button = QPushButton()
        self.add_images_button.setObjectName(u"add_images_button")
        self.add_images_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.add_images_button.setText('Add')
        self.add_images_button.clicked.connect(self.pick_files)
        # sort button
        self.sort_images_button = QPushButton()
        self.sort_images_button.setObjectName(u"sort_images_button")
        self.sort_images_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.sort_images_button.setText('Sort')
        self.sort_images_button.clicked.connect(self.pick_sort_type)
        # remove button
        self.remove_images_button = QPushButton()
        self.remove_images_button.setObjectName(u"remove_images_button")
        self.remove_images_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.remove_images_button.setText('Remove')
        self.remove_images_button.clicked.connect(self.remove_image)
        # adding buttons to h_layout
        self.images_buttons_h_box.addWidget(self.add_images_button)
        self.images_buttons_h_box.addWidget(self.sort_images_button)
        self.images_buttons_h_box.addWidget(self.remove_images_button)

        # Adding widgets to left part
        self.addWidget(self.images_label)
        self.addWidget(self.images_list)
        self.addWidget(self.images_count_label)
        self.addWidget(self.properties_list)
        self.addLayout(self.images_buttons_h_box)

    # Open file dialog
    def pick_files(self):
        file_dialog = QFileDialog()
        selected_files = file_dialog.getOpenFileNames(self.root_widget, 'Open files',
                                                      str(Path.home()),
                                                      'Image files (*.png *.jpg *.jpeg *.tif *.gif)', )
        selected_files = selected_files[0]

        if selected_files:
            for file in selected_files:
                if os.path.isfile(file):
                    if file not in self.list_of_images:
                        self.list_of_images.append(file)

        self.sort_images()
        self.images_count_label.setText(f'Images count: {len(self.list_of_images)}')

    # Display properties of the image
    def generate_image_properties(self, file_name):
        ignored_exif_list = [
            'ImageWidth',
            'ImageHeight',
            'TileOffsets',
            'TileByteCounts',
            'StripByteCounts',
            'StripOffsets',
        ]
        try:
            img = Image.open(file_name)
        except FileNotFoundError:
            return
        width, height = img.size
        file_base_name = basename(file_name)
        image_file_size = file_size(file_name)
        image_ext = img.format
        modification_time = os.path.getmtime(file_name)
        modification_time = datetime.fromtimestamp(modification_time).strftime('%Y-%m-%d %H:%M:%S')
        image_mode = img.mode
        mode_to_bpp = {'1': 1, 'L': 8, 'P': 8, 'RGB': 24, 'RGBA': 32, 'CMYK': 32, 'YCbCr': 24, 'I': 32, 'F': 32}

        exif_tags = {}

        for k, v in img.getexif().items():
            tag = TAGS.get(k)
            exif_tags[tag] = v

        self.properties_list.clear()
        self.properties_list.addItem(f'Filename: {file_base_name}')
        self.properties_list.addItem(f'File size: {image_file_size}')
        self.properties_list.addItem(f'Modified: {modification_time}')
        self.properties_list.addItem(f'Format: {image_ext}')
        self.properties_list.addItem(f'Width: {width}')
        self.properties_list.addItem(f'Height: {height}')
        self.properties_list.addItem(f'Mode: {image_mode}')
        try:
            self.properties_list.addItem(f'Bit depth: {mode_to_bpp[image_mode]}')
        except KeyError:
            pass
        # EXIF
        for exif_tag in exif_tags:
            if exif_tag not in ignored_exif_list:
                pattern = re.compile(r'((?<=[^\W[A-Z])[A-Z]|(?<=\S)[A-Z](?=[a-z]))')
                exif_tag_split = pattern.sub(r' \1', str(exif_tag))
                self.properties_list.addItem(f'{exif_tag_split}: {exif_tags[exif_tag]}')

    def pick_sort_type(self):
        new_task_dialog = SortImagesDialog(self.sort_type)
        if new_task_dialog.exec_() == QDialog.Accepted:
            if new_task_dialog.sort_picker.currentText() == 'unsorted':
                self.sort_type = Sort.UNSORTED
            elif new_task_dialog.sort_picker.currentText() == 'alphabetically':
                self.sort_type = Sort.ALPHABETICALLY
            elif new_task_dialog.sort_picker.currentText() == 'types':
                self.sort_type = Sort.TYPES
        else:
            pass

        self.sort_images()

    # Sorting images
    def sort_images(self):
        new_images_list = []
        if self.sort_type == Sort.UNSORTED:
            new_images_list = self.list_of_images
        elif self.sort_type == Sort.ALPHABETICALLY:
            new_images_list = self.list_of_images
            new_images_list = natsorted(new_images_list)

        self.images_list.clear()
        self.images_list.addItems(new_images_list)

    # Preview image
    def remove_image(self, remove_from_disk=False, item=None):
        # If item specified
        if item:
            index = self.images_list.row(item)
            self.images_list.takeItem(index)
            self.list_of_images.remove(item.text())
            if remove_from_disk:
                os.remove(item.text())
            try:
                self.images_list.setCurrentRow(index)
            except IndexError:
                pass

            if self.images_list.count() == 0:
                self.properties_list.clear()

            return

        # If item not specified
        for selected_image in self.images_list.selectedItems():
            index = self.images_list.row(selected_image)
            self.images_list.takeItem(index)
            self.list_of_images.remove(selected_image.text())
            if remove_from_disk:
                os.remove(selected_image.text())
            try:
                self.images_list.setCurrentRow(index)
            except IndexError:
                pass

            if self.images_list.count() == 0:
                self.properties_list.clear()

        self.images_count_label.setText(f'Images count: {len(self.list_of_images)}')



from os.path import basename

import cv2
import numpy as np

from imgmanip.functions.get_ext import get_ext_from_file
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import ColorDetectionTask


def detect_color(image, task: ColorDetectionTask, save_type: SaveType, out_path=None):
    img = cv2.imread(image)
    # hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    # From color
    lower = np.array(task.color, dtype="uint8")
    # To color
    upper = np.array(task.color, dtype="uint8")
    # Mask with color
    mask = cv2.inRange(img, lower, upper)
    image_ext = get_ext_from_file(image)

    if save_type == SaveType.SELECT_PATH:
        new_file_name = out_path + '/' + basename(image)
        mask_file_name = out_path + '/' + basename(image).replace(image_ext, f'_mask{image_ext}')
    elif save_type == SaveType.IMAGE_PATH:
        new_file_name = image.replace(image_ext, f'_new{image_ext}')
        mask_file_name = image.replace(image_ext, f'_mask{image_ext}')
    else:
        new_file_name = image
        mask_file_name = image.replace(image_ext, f'_mask{image_ext}')

    # Saving mask
    if task.save_mask and save_type.OVERWRITE:
        cv2.imwrite(mask_file_name, mask)

    contours, hierarchy = cv2.findContours(
        mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    cv2.drawContours(img, contours, -1, (0, 255, 0), 5)

    cv2.imwrite(new_file_name, img)
    return new_file_name



from PySide6.QtWidgets import QFrame, QVBoxLayout, QLayout, QLabel, QComboBox

from imgmanip.models.axis import Axis
from imgmanip.models.task import FlipTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class FlipTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == FlipTask:
            print(task.axis.value)
            self.task = task
        else:
            self.task = FlipTask(axis=Axis.HORIZONTAL)

        self.flip_box = QVBoxLayout()

        self.description_label = QLabel()
        self.description_label.setText("This task flips the image in horizontal or vertical axis.")

        self.flip_box.setObjectName(u"flip_box")
        self.flip_box.setSizeConstraint(QLayout.SetDefaultConstraint)

        self.axis_picker = QComboBox()

        for axis in Axis:
            self.axis_picker.addItem(axis.value)

        self.axis_picker.setCurrentText(self.task.axis.value)
        self.axis_picker.currentTextChanged.connect(self.change_values)

        self.flip_box.addWidget(self.description_label)
        self.flip_box.addWidget(HorizontalLine())
        self.flip_box.addWidget(self.axis_picker)

        self.setLayout(self.flip_box)
        self.hide()

    # Change values
    def change_values(self):
        self.task.axis = Axis(self.axis_picker.currentText())



from imgmanip.functions.tasks.color_detection import detect_color
from imgmanip.functions.tasks.compress import compress_image
from imgmanip.functions.tasks.convert import convert_image
from imgmanip.functions.tasks.convert_color_mode import convert_color_mode_image
from imgmanip.functions.tasks.flip import flip_image
from imgmanip.functions.tasks.invert import invert_image
from imgmanip.functions.tasks.resize import resize_image
from imgmanip.models.task import InvertTask, ResizeTask, FlipTask, ConvertTask, CompressTask, ColorDetectionTask


def run_task(index, images_list, list_of_tasks, save_type, out_path):
    image = images_list[index]
    tasks_functions = {
        'resize': resize_image,
        'invert': invert_image,
        'flip': flip_image,
        'convert': convert_image,
        'compress': compress_image,
        'color_detection': detect_color,
        'convert_color_mode': convert_color_mode_image,
    }

    for task in list_of_tasks:
        try:
            file_name = tasks_functions[task.name](image, task, save_type, out_path)
            image = file_name
        except Exception as e:
            print(e)
            return image



from PySide6.QtWidgets import QFrame, QVBoxLayout, QCheckBox, QLayout, QLabel, QPushButton, QColorDialog, QComboBox

from imgmanip.models.color_mode import ColorMode
from imgmanip.models.task import ConvertColorModeTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class ConvertColorModeTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == ConvertColorModeTask:
            self.task = task
        else:
            self.task = ConvertColorModeTask(color_mode=ColorMode.RGB)

        self.description_label = QLabel()
        self.description_label.setText(
            "This task marks convert the image to selected color mode."
        )

        self.color_mode_box = QVBoxLayout()
        self.color_mode_box.setObjectName(u"color_mode_box")
        self.color_mode_box.setSizeConstraint(QLayout.SetDefaultConstraint)

        self.color_mode_picker = QComboBox(self)
        self.color_mode_picker.addItems([color_mode.value for color_mode in ColorMode])
        self.color_mode_picker.setCurrentText(self.task.color_mode.value)
        self.color_mode_picker.currentTextChanged.connect(self.change_values)

        self.color_mode_box.addWidget(self.description_label)
        self.color_mode_box.addWidget(HorizontalLine())
        self.color_mode_box.addWidget(self.color_mode_picker)

        self.setLayout(self.color_mode_box)
        self.hide()

    # Change values
    def change_values(self):
        self.task.color_mode = ColorMode(self.color_mode_picker.currentText())



def create_time_str(str_to_create, seconds):
    str_to_create = str_to_create
    minutes, seconds = divmod(round(seconds, 2), 60)
    seconds = round(seconds, 2)
    hours, minutes = divmod(minutes, 60)
    if hours > 0:
        if hours == 1:
            str_to_create += f' {hours} hour'
        else:
            str_to_create += f' {hours} hours'

    if minutes > 0:
        if minutes == 1:
            str_to_create += f' {minutes} minute'
        else:
            str_to_create += f' {minutes} minutes'

    if minutes == 1:
        str_to_create += f' {seconds} second'
    else:
        str_to_create += f' {seconds} seconds'

    return str_to_create



from PySide6.QtCore import Qt
from PySide6.QtGui import QCursor
from PySide6.QtWidgets import QVBoxLayout, QPushButton, QProgressBar, QHBoxLayout, QGraphicsPixmapItem, QGraphicsView, \
    QGraphicsScene


class CenterPart(QVBoxLayout):
    def __init__(self, root_widget):
        super().__init__()
        self.root_widget = root_widget

        self.setObjectName(u"left_vbox")

        # Image preview with scrolling, zooming
        self.image_preview_scene = QGraphicsScene()
        self.image_preview_view = QGraphicsView(self.image_preview_scene)
        self.image_preview_view.setMinimumSize(600, 600)
        # self.image_preview_view.setMaximumSize(1200, 800)
        # self.image_preview_view.contextMenuEvent = self.image_preview_menu
        self.image_preview_view.wheelEvent = self.wheel_event
        self.image_preview = QGraphicsPixmapItem()
        # self.image_preview.setPixmap(QPixmap.fromImage(QImage('assets/logo.png')))
        self.image_preview_scene.addItem(self.image_preview)
        self.image_preview_view.show()

        # ProgressBar
        self.progress = QProgressBar()
        self.progress.setValue(0)
        self.progress.setVisible(False)

        # Bottom Horizontal box
        self.bottom_center_h_box = QHBoxLayout()
        self.bottom_center_h_box.setObjectName(u"bottom_center_h_box")
        # Start Button
        self.start_button = QPushButton()
        self.start_button.setObjectName(u"start_button")
        self.start_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.start_button.setText('Start')
        self.start_button.clicked.connect(self.root_widget.start_tasks)
        self.start_button.setMaximumWidth(100)
        self.start_button.setStyleSheet("background-color: #99A2FF; color: #111111; font-weight: bold")

        # Fit in view button
        self.fit_in_view_button = QPushButton()
        self.fit_in_view_button.setText('Fit in view')
        self.fit_in_view_button.setMaximumWidth(100)
        self.fit_in_view_button.clicked.connect(self.fit_in_view)

        self.bottom_center_h_box.addWidget(self.fit_in_view_button, alignment=Qt.AlignLeft)
        self.bottom_center_h_box.addWidget(self.start_button)
        # self.addWidget(self.image_preview)
        self.addWidget(self.image_preview_view)
        self.addWidget(self.progress)
        self.addLayout(self.bottom_center_h_box)

    # Zooming event
    def wheel_event(self, event):
        if event.angleDelta().y() > 0:
            zoom = 1.2
        else:
            zoom = .8
        if event.modifiers() == Qt.CTRL:
            self.image_preview_view.scale(zoom, zoom)
        elif event.modifiers() == Qt.SHIFT:
            hor_scroll_value = self.image_preview_view.horizontalScrollBar().value() + -1 * event.angleDelta().y()
            self.image_preview_view.horizontalScrollBar().setValue(hor_scroll_value)
        else:
            ver_scroll_value = self.image_preview_view.verticalScrollBar().value() + -1 * event.angleDelta().y()
            self.image_preview_view.verticalScrollBar().setValue(ver_scroll_value)

    # Fit in view image preview
    def fit_in_view(self):
        self.image_preview_view.fitInView(self.image_preview, Qt.KeepAspectRatio)



from PySide6.QtWidgets import QFrame


class HorizontalLine(QFrame):
    def __init__(self):
        super().__init__()
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)



from PySide6.QtWidgets import QFrame, QVBoxLayout, QCheckBox, QLayout, QLabel, QPushButton, QColorDialog

from imgmanip.models.task import ColorDetectionTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class ColorDetectionTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == ColorDetectionTask:
            self.task = task
        else:
            self.task = ColorDetectionTask(save_mask=True)

        self.description_label = QLabel()
        self.description_label.setText(
            "This task marks where the given color appears in the image."
            "\n"
            "Additionally, it can save the mask in .png format, "
            "\n"
            "shapefile and geojson file."
        )

        self.color_detection_frame = QFrame()
        self.color_detection_box = QVBoxLayout()
        self.color_detection_box.setObjectName(u"color_detection_box")
        self.color_detection_box.setSizeConstraint(QLayout.SetDefaultConstraint)
        # Color picker
        self.color_picker_button = QPushButton()
        self.color_picker_button.setText('Pick color')
        self.color_picker_button.clicked.connect(self.color_picker)

        if task and type(task) == ColorDetectionTask:
            if task.color:
                self.color_picker_button.setText(f'RGB: {task.color}')

        # Save mask
        self.save_mask_checkbox = QCheckBox()
        self.save_mask_checkbox.setText('Save image mask')
        self.save_mask_checkbox.setChecked(self.task.save_mask)
        self.save_mask_checkbox.stateChanged.connect(self.change_values)

        self.color_detection_box.addWidget(self.description_label)
        self.color_detection_box.addWidget(HorizontalLine())
        self.color_detection_box.addWidget(self.color_picker_button)
        self.color_detection_box.addWidget(self.save_mask_checkbox)
        self.setLayout(self.color_detection_box)
        self.hide()

    # Change values
    def change_values(self):
        self.task.save_mask = self.save_mask_checkbox.isChecked()

    # Pick color
    def color_picker(self):
        color = QColorDialog.getColor()
        selected_color = [color.getRgb()[0], color.getRgb()[1], color.getRgb()[2]]
        self.color_picker_button.setText(f'RGB: {selected_color}')
        self.task.color = selected_color



import cv2
from PIL import Image

from imgmanip.dialogs.confirm_dialog import ConfirmDialog
from imgmanip.functions.save_img import save_image_path
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import InvertTask


def invert_image(image, invert_task: InvertTask, save_type: SaveType, out_path=None):
    img = cv2.imread(image)
    inverted_image = cv2.bitwise_not(img)

    new_file_name = save_image_path(image_path=image, save_type=save_type, out_path=out_path)

    cv2.imwrite(new_file_name, inverted_image)
    return new_file_name



import os
import sys
from imgmanip import config

from PySide6 import QtGui
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QMainWindow, QApplication

from imgmanip.widgets.main_screen import UiMainWindow
import qdarktheme

os.environ['QT_IMAGEIO_MAXALLOC'] = "10000000000000000000000000000000000000000000000000000000000000000"
QtGui.QImageReader.setAllocationLimit(0)


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.setMouseTracking(True)
        self.config = config.read_config()

        if self.config['maximized_window']:
            self.showMaximized()
        self.ui = UiMainWindow(self, self.config)


def main():
    app = QApplication(sys.argv)
    app.setStyleSheet(qdarktheme.load_stylesheet())
    app.setAttribute(Qt.AA_DontUseNativeDialogs)

    window = MainWindow()
    window.show()

    sys.exit(app.exec())



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QFrame, QVBoxLayout, QLayout, QLabel, QHBoxLayout, QSlider

from imgmanip.models.task import CompressTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class CompressTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == CompressTask:
            self.task = task
        else:
            self.task = CompressTask(quality=95)

        self.compress_box = QVBoxLayout()

        self.description_label = QLabel()
        self.description_label.setText(
            "This task compresses the image. The lower the 'quality',\nthe smaller the file size.")

        self.compress_box.setObjectName(u"compress_box")
        self.compress_box.setSizeConstraint(QLayout.SetDefaultConstraint)
        # Label + slider box
        self.compress_quality_box = QHBoxLayout()
        # compress quality label
        self.compress_quality_label = QLabel()
        self.compress_quality_label.setText('Quality')
        # compress quality slider label
        self.compress_quality_value_label = QLabel()
        # compress quality slider
        self.compress_quality_slider = QSlider(Qt.Horizontal)
        self.compress_quality_slider.setMinimum(1)
        self.compress_quality_slider.setMaximum(100)
        self.compress_quality_slider.setValue(self.task.quality)
        self.compress_quality_slider.setTickPosition(QSlider.TicksBelow)
        self.compress_quality_slider.setTickInterval(5)
        self.compress_quality_slider.valueChanged.connect(self.update_slider_value)
        self.compress_quality_value_label.setText(f'{self.compress_quality_slider.value()}%')
        self.compress_quality_value_label.setFixedWidth(50)

        self.compress_quality_box.addWidget(self.compress_quality_value_label)
        self.compress_quality_box.addWidget(self.compress_quality_slider)

        self.compress_box.addWidget(self.description_label)
        self.compress_box.addWidget(HorizontalLine())
        self.compress_box.addWidget(self.compress_quality_label)
        self.compress_box.addLayout(self.compress_quality_box)
        self.setLayout(self.compress_box)
        self.hide()

    # Compress slider value
    def update_slider_value(self):
        self.compress_quality_value_label.setText(f'{self.compress_quality_slider.value()}%')
        self.task.quality = self.compress_quality_slider.value()



import multiprocessing
import os
import sys
import time
from pathlib import Path
from imgmanip import config

from functools import partial
import webbrowser

import pyperclip
from PIL import Image
from PySide6 import QtCore, QtWidgets
from PySide6.QtCore import (QMetaObject, Qt)
from PySide6.QtGui import (QPixmap, QAction)
from PySide6.QtWidgets import (QGridLayout, QHBoxLayout, QWidget,
                               QDialog, QMenuBar, QFileDialog)

from imgmanip.dialogs.confirm_dialog import ConfirmDialog
from imgmanip.dialogs.info_dialog import InfoDialog
from imgmanip.dialogs.preferences_dialog import PreferencesDialog
from imgmanip.dialogs.rename_dialog import RenameDialog
from imgmanip.functions.create_time_str import create_time_str
from imgmanip.functions.run_tasks import run_task
from imgmanip.logs import save_images_names_to_txt
from imgmanip.models.save_type import SaveType
from imgmanip.widgets.main.center_part import CenterPart
from imgmanip.widgets.main.left_part import LeftPart
from imgmanip.widgets.main.right_part import RightPart

Image.MAX_IMAGE_PIXELS = 933120000
cpu_count = multiprocessing.cpu_count()

platform_path = '/'
if sys.platform == 'win32':
    platform_path = '\\'


class UiMainWindow(QWidget):
    def __init__(self, main_window, config_dict):
        # Main window
        super().__init__()
        if not main_window.objectName():
            main_window.setObjectName(u"MainWindow")
        main_window.resize(1000, 600)
        main_window.setWindowTitle('IMGManip')
        self.config = config_dict

        # main_window.setMaximumHeight(600)
        self.main_window = main_window

        # Creating FILE actions
        self.preferences_action = QAction("&Preferences", self)
        self.preferences_action.triggered.connect(self.open_preferences)
        self.exit_action = QAction("&Exit", self)
        self.exit_action.triggered.connect(self.exit_tool)
        # Creating EDIT actions
        self.fit_in_view_action = QAction("&Fit in view", self, checkable=True)
        self.fit_in_view_action.setChecked(True)

        self.create_menu_bar()

        # Main central widget
        self.central_widget = QWidget(main_window)
        self.central_widget.setObjectName(u"central_widget")
        self.gridLayout = QGridLayout(self.central_widget)
        self.gridLayout.setObjectName(u"gridLayout")

        # Horizontal layout split on 3 parts
        self.main_h_layout = QHBoxLayout()
        self.main_h_layout.setObjectName(u"main_h_layout")

        self.left_part = LeftPart(self)
        self.left_part.images_list.installEventFilter(self)
        # self.left_part.images_list.itemClicked.connect(self.preview_image)
        self.left_part.images_list.itemActivated.connect(self.preview_image)
        self.left_part.properties_list.installEventFilter(self)

        self.center_part = CenterPart(self)
        self.right_part = RightPart(self)
        # Adding parts to the main widget (main_h_layout)
        self.main_h_layout.addLayout(self.left_part)
        self.main_h_layout.addLayout(self.center_part)
        self.main_h_layout.addLayout(self.right_part)

        self.gridLayout.addLayout(self.main_h_layout, 0, 0, 1, 1)

        main_window.setCentralWidget(self.central_widget)

        QMetaObject.connectSlotsByName(main_window)

    # Create menu bar
    def create_menu_bar(self):
        menu_bar = QMenuBar(self)
        # Creating menus
        file_menu = menu_bar.addMenu("&File")
        view_menu = menu_bar.addMenu("&View")

        file_menu.addAction(self.preferences_action)
        file_menu.addAction(self.exit_action)
        view_menu.addAction(self.fit_in_view_action)

        self.main_window.setMenuBar(menu_bar)

    # Open preferences dialog
    def open_preferences(self):
        preferences_dialog = PreferencesDialog(self.config)
        if preferences_dialog.exec_() == QDialog.Accepted:
            self.config = preferences_dialog.config
            config.write_config(preferences_dialog.config)
            self.left_part.sort_images()
            if self.config['maximized_window']:
                self.main_window.showMaximized()
            else:
                self.main_window.showNormal()

    # Exit tool
    @staticmethod
    def exit_tool():
        sys.exit()

    # Menus on right click
    def eventFilter(self, source, event):
        if (event.type() == QtCore.QEvent.ContextMenu and
                source is self.left_part.images_list):

            # Menu for every item
            menu = QtWidgets.QMenu()
            menu.addAction('Copy path')
            menu.addAction('Open containing folder')
            menu.addAction('Rename')
            menu.addAction('Remove')
            menu.addAction('Remove from disk')

            if source.itemAt(event.pos()):
                action = menu.exec_(event.globalPos())
                if action:
                    item = source.itemAt(event.pos())
                    if item:
                        # COPY PATH
                        if action.toolTip() == 'Copy path':
                            pyperclip.copy(item.text())
                        # OPEN CONTAINING FOLDER
                        elif action.toolTip() == 'Open containing folder':
                            webbrowser.open(os.path.dirname(item.text()))
                        # RENAME IMAGE FILE
                        elif action.toolTip() == 'Rename':
                            rename_dialog = RenameDialog(old_name=item.text())
                            if rename_dialog.exec_() == QDialog.Accepted:
                                new_name = rename_dialog.new_name_input.text()
                                self.left_part.list_of_images[
                                    self.left_part.list_of_images.index(item.text())] = new_name
                                os.rename(item.text(), new_name)
                                self.left_part.sort_images()
                                self.preview_image(self.left_part.images_list.findItems(new_name, Qt.MatchContains)[0])
                            else:
                                rename_dialog.close()

                        # REMOVE IMAGE FROM LIST
                        elif action.toolTip() == 'Remove':
                            self.left_part.remove_image(False, item)

                        # REMOVE IMAGE FROM DISK
                        elif action.toolTip() == 'Remove from disk':
                            confirm_delete = ConfirmDialog(title='Remove file from disk', desc='Are you sure?')
                            if confirm_delete.exec_() == QDialog.Accepted:
                                self.left_part.remove_image(True, item)
                            else:
                                confirm_delete.close()
                return True

        if (event.type() == QtCore.QEvent.ContextMenu and
                source is self.left_part.properties_list):
            # Menu for every item
            menu = QtWidgets.QMenu()
            menu.addAction('Copy')
            menu.addAction('Copy value')
            menu.addAction('Copy property name')
            if source.itemAt(event.pos()):
                action = menu.exec_(event.globalPos())
                if action:
                    item = source.itemAt(event.pos())
                    if action.toolTip() == "Copy":
                        pyperclip.copy(item.text())
                    elif action.toolTip() == "Copy value":
                        pyperclip.copy(item.text().split(': ')[-1])
                    elif action.toolTip() == "Copy property name":
                        pyperclip.copy(item.text().split(': ')[0])

        return super(UiMainWindow, self).eventFilter(source, event)

    # Preview image
    def preview_image(self, item):
        file_name = item.text()
        if file_name:
            try:
                pix = QPixmap(file_name)
                self.left_part.generate_image_properties(file_name)
            except IndexError or TypeError:
                pix = QPixmap(self.left_part.images_list.item(0).text())
                self.left_part.generate_image_properties(self.left_part.images_list.item(0).text())
        else:
            try:
                pix = QPixmap(self.left_part.images_list.selectedItems()[0].text())
                self.left_part.generate_image_properties(self.left_part.images_list.selectedItems()[0].text())
            except IndexError:
                pix = QPixmap(self.left_part.images_list.item(0).text())
                self.left_part.generate_image_properties(self.left_part.images_list.item(0).text())

        pix_size = pix.size()

        self.center_part.image_preview.setPixmap(
            pix.scaled(pix_size)
        )

        if self.fit_in_view_action.isChecked():
            self.center_part.fit_in_view()

        self.center_part.image_preview_scene.setSceneRect(0, 0, pix_size.toTuple()[0],
                                                          pix_size.toTuple()[1])

    # Start tasks
    def start_tasks(self):
        start_time = time.time()

        # If no image added
        if not self.left_part.list_of_images:
            error_dialog = InfoDialog(title='Start tasks',
                                      text='You need to select at least one image.')
            if error_dialog.exec_() == QDialog.Accepted:
                return
        # If no task added
        if not self.right_part.list_of_tasks:
            error_dialog = InfoDialog(title='Start tasks',
                                      text='You need to select at least one task.')
            if error_dialog.exec_() == QDialog.Accepted:
                return

        # If no path selected
        if not os.path.exists(
                self.right_part.path_input.text()) \
                and \
                self.right_part.save_type_picker.currentText() == SaveType.SELECT_PATH.value:

            error_dialog = InfoDialog(title='Start tasks',
                                      text='You need to select valid path.')
            if error_dialog.exec_() == QDialog.Accepted:
                return
        # Overwrite warning
        if self.right_part.save_type_picker.currentText() == SaveType.OVERWRITE.value:
            confirm_delete = ConfirmDialog(title='Start tasks',
                                           desc='This process will overwrite all selected files!\nAre you sure?')
            if confirm_delete.exec_() == QDialog.Accepted:
                confirm_delete.close()
            else:
                confirm_delete.close()
                return
        self.center_part.start_button.setText('Stop')
        self.center_part.progress.setVisible(True)

        # try:
        #     self.left_part.images_list.setCurrentRow(0)
        #     self.preview_image(self.left_part.images_list.item(0).text())
        # except IndexError:
        #     pass

        images_list = [self.left_part.images_list.item(index).text() for index in
                       range(self.left_part.images_list.count())]

        # Multiprocessing
        pool = multiprocessing.Pool(processes=self.config['cpu_count'])
        run_func = partial(run_task, images_list=images_list,
                           list_of_tasks=self.right_part.list_of_tasks,
                           save_type=SaveType(self.right_part.save_type_picker.currentText()),
                           out_path=self.right_part.path_input.text())
        jobs = []
        job_count = 0
        images_count = len(images_list)
        for job in pool.imap_unordered(run_func, range(images_count)):
            jobs.append(job)
            job_count += 1
            progress = round((job_count / len(images_list)) * 100, 1)
            self.center_part.progress.setValue(progress)
            self.center_part.progress.setFormat(f'{progress}% {job_count}/{images_count}')

        pool.close()
        end_time = time.time()

        error_images = [ele for ele in jobs if ele is not None]
        if error_images:
            save_log_dialog = ConfirmDialog(title=f'Could not process {len(error_images)} files',
                                            desc=f"Could not process {len(error_images)} files. \n"
                                                 "Do you want to save a text file with the names of this files?")
            if save_log_dialog.exec_() == QDialog.Accepted:
                txt_file = QFileDialog.getSaveFileName(self, 'Save text file', str(Path.home()) + '/error-images.txt',
                                                       'Text file (*.txt)')
                if txt_file[0] != '':
                    save_images_names_to_txt(error_images, txt_file[0])

        self.center_part.progress.setStyleSheet('color: white')
        self.center_part.progress.setVisible(False)
        self.center_part.start_button.setText('Start')

        done_text = f'Done in:'
        done_text = create_time_str(done_text, end_time - start_time)

        time_dialog = InfoDialog(title='Done', text=done_text)
        time_dialog.exec()



from PIL import Image, ImageOps

from imgmanip.functions.save_img import save_image_path
from imgmanip.models.axis import Axis
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import FlipTask


def flip_image(image, flip_task: FlipTask, save_type: SaveType, out_path=None):
    img = Image.open(image)
    if flip_task.axis == Axis.HORIZONTAL:
        img = ImageOps.mirror(img)
    elif flip_task.axis == Axis.VERTICAL:
        img = ImageOps.flip(img)

    new_file_name = save_image_path(image_path=image, save_type=save_type, out_path=out_path)

    img.save(new_file_name)
    return new_file_name



from PySide6.QtWidgets import QLayout, QVBoxLayout, QFrame, QComboBox, QLabel

from imgmanip.models.image_extension import ImageExtension
from imgmanip.models.task import ConvertTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class ConvertTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == ConvertTask:
            self.task = task
        else:
            self.task = ConvertTask(convert_ext=ImageExtension.PNG)

        self.convert_box = QVBoxLayout()

        self.description_label = QLabel()
        self.description_label.setText(
            "This task converts the image to the other format.")

        self.convert_box.setObjectName(u"convert_box")
        self.convert_box.setSizeConstraint(QLayout.SetDefaultConstraint)
        # Extension picker label
        self.extension_picker_label = QLabel()
        self.extension_picker_label.setText("New file extension")
        # Extension picker
        self.extension_picker = QComboBox(self)
        self.extension_picker.addItems([ext.value for ext in ImageExtension])
        self.extension_picker.setCurrentText(self.task.convert_ext.value)
        self.extension_picker.currentTextChanged.connect(self.change_values)

        self.extension_picker.setObjectName(u"extension_picker")

        # self.extension_picker.currentTextChanged.connect(self.change_extension)
        self.convert_box.addWidget(self.description_label)
        self.convert_box.addWidget(HorizontalLine())
        self.convert_box.addWidget(self.extension_picker_label)
        self.convert_box.addWidget(self.extension_picker)
        self.setLayout(self.convert_box)
        self.hide()

    # Change values
    def change_values(self):
        self.task.convert_ext = ImageExtension(self.extension_picker.currentText())



import os


def convert_bytes(num):
    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0


def file_size(file_path):
    if os.path.isfile(file_path):
        file_info = os.stat(file_path)
        return convert_bytes(file_info.st_size)



from PySide6.QtCore import (QMetaObject, Qt)
from PySide6.QtWidgets import (QDialog, QDialogButtonBox,
                               QSizePolicy, QGridLayout, QVBoxLayout, QLayout, QLabel)


class InfoDialog(QDialog):
    def __init__(self, title: str, text: str):
        super().__init__()
        if not self.objectName():
            self.setObjectName(u"self")

        self.setWindowTitle(title)

        self.grid_layout = QGridLayout(self)
        self.grid_layout.setObjectName(u"grid_layout")

        # Main layout
        self.vertical_layout = QVBoxLayout()
        self.vertical_layout.setObjectName(u"vertical_layout")
        self.vertical_layout.setSizeConstraint(QLayout.SetDefaultConstraint)

        # Error label
        self.error_label = QLabel()
        self.error_label.setText(text)
        self.vertical_layout.addWidget(self.error_label)

        self.button_box = QDialogButtonBox(self)
        self.button_box.setObjectName(u"button_box")
        size_policy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        size_policy3.setHorizontalStretch(0)
        size_policy3.setVerticalStretch(0)
        size_policy3.setHeightForWidth(self.button_box.sizePolicy().hasHeightForWidth())
        self.button_box.setSizePolicy(size_policy3)
        self.button_box.setOrientation(Qt.Horizontal)
        self.button_box.setStandardButtons(QDialogButtonBox.Ok)
        self.button_box.button(QDialogButtonBox.Ok).setStyleSheet('background-color: #99A2FF; color: #111111;')
        self.button_box.setCenterButtons(True)

        self.grid_layout.addLayout(self.vertical_layout, 0, 0, 1, 1)
        self.button_box.accepted.connect(self.accept)

        self.grid_layout.addWidget(self.button_box, 1, 0, 1, 1)

        QMetaObject.connectSlotsByName(self)



from PySide6 import QtWidgets
from PySide6.QtGui import QCursor, Qt, QFont
from PySide6.QtWidgets import QLabel, QVBoxLayout, QListWidget, QHBoxLayout, QPushButton, QDialog, QComboBox, \
    QFrame, QLineEdit, QFileDialog

from imgmanip.dialogs.new_task_dialog import NewTaskDialog
from imgmanip.models.save_type import SaveType


class RightPart(QVBoxLayout):
    def __init__(self, root_widget):
        # Right part

        super().__init__()
        self.list_of_tasks = []

        self.setObjectName(u"right_vbox")
        self.root_widget = root_widget

        self.default_list_font = QFont()
        self.default_list_font.setPixelSize(13)

        # "Tasks" text
        self.tasks_label = QLabel()
        self.tasks_label.setObjectName(u"tasks_label")
        self.tasks_label.setText('Tasks')

        # List of the created tasks
        self.tasks_list = QListWidget()
        self.tasks_list.setObjectName(u"tasks_list")
        self.tasks_list.setMaximumWidth(300)

        # Horizontal layout with add and remove button (TASKS)
        self.tasks_buttons_h_box = QHBoxLayout()
        self.tasks_buttons_h_box.setObjectName(u"tasks_buttons_h_box")
        # add button
        self.add_task_button = QPushButton()
        self.add_task_button.setObjectName(u"add_task_button")
        self.add_task_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.add_task_button.setText("Add")
        self.add_task_button.clicked.connect(self.add_task)
        # Edit button
        self.edit_task_button = QPushButton()
        self.edit_task_button.setObjectName(u"edit_task_button")
        self.edit_task_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.edit_task_button.setText("Edit")
        self.edit_task_button.clicked.connect(self.edit_task)
        # remove button
        self.remove_task_button = QPushButton()
        self.remove_task_button.setObjectName(u"remove_task_button")
        self.remove_task_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.remove_task_button.setText("Remove")
        self.remove_task_button.clicked.connect(self.remove_task)
        # adding button to horizontal box
        self.tasks_buttons_h_box.addWidget(self.add_task_button)
        self.tasks_buttons_h_box.addWidget(self.edit_task_button)
        self.tasks_buttons_h_box.addWidget(self.remove_task_button)

        # Save type label
        self.save_path_label = QLabel()
        self.save_path_label.setText('Save path')

        # Save type picker
        self.save_type_picker = QComboBox()
        for save_type in SaveType:
            self.save_type_picker.addItem(save_type.value)
        self.save_type_picker.currentTextChanged.connect(self.change_save_type)

        self.path_select_box = QFrame()
        self.path_select_box.setMaximumWidth(300)
        self.path_select_h_box = QHBoxLayout()
        self.path_select_h_box.setContentsMargins(0, 0, 0, 0)
        self.path_input = QLineEdit()
        self.path_input.setObjectName(u"path_label")
        self.path_input.setText('None')

        self.path_pick_button = QPushButton()
        self.path_pick_button.setText('Select path')
        self.path_pick_button.clicked.connect(self.select_save_path)

        self.path_select_h_box.addWidget(self.path_input)
        self.path_select_h_box.addWidget(self.path_pick_button)
        self.path_select_box.setLayout(self.path_select_h_box)

        self.change_order_box = QFrame()
        self.change_order_box.setMaximumWidth(300)
        self.change_order_h_box = QHBoxLayout()
        self.change_order_h_box.setContentsMargins(0, 0, 0, 0)
        self.up_button = QPushButton()
        self.up_button.setText('↑')
        self.up_button.clicked.connect(self.move_task_up)
        self.down_button = QPushButton()
        self.down_button.setText('↓')
        self.down_button.clicked.connect(self.move_task_down)
        self.change_order_h_box.addWidget(self.up_button)
        self.change_order_h_box.addWidget(self.down_button)
        self.change_order_box.setLayout(self.change_order_h_box)

        # Adding widgets to right part
        self.addWidget(self.tasks_label)
        self.addWidget(self.tasks_list)
        self.addWidget(self.change_order_box)
        self.addWidget(self.save_path_label)
        self.addWidget(self.save_type_picker)
        self.addWidget(self.path_select_box)
        self.addLayout(self.tasks_buttons_h_box)

        self.change_save_type()

    # Move up
    def move_task_up(self):
        for selected_index in self.tasks_list.selectedIndexes():
            selected_index = selected_index.row()
            if selected_index > 0:
                self.list_of_tasks.insert(selected_index - 1, self.list_of_tasks.pop(selected_index))
                self.generate_list_of_tasks()
                self.tasks_list.setCurrentRow(selected_index - 1)
            else:
                self.generate_list_of_tasks()
                self.tasks_list.setCurrentRow(selected_index)

    # Move down
    def move_task_down(self):
        for selected_index in self.tasks_list.selectedIndexes():
            selected_index = selected_index.row()
            if selected_index < self.tasks_list.count() - 1:
                self.list_of_tasks.insert(selected_index + 1, self.list_of_tasks.pop(selected_index))
                self.generate_list_of_tasks()
                self.tasks_list.setCurrentRow(selected_index + 1)
            else:
                self.generate_list_of_tasks()
                self.tasks_list.setCurrentRow(selected_index)

    # Save type
    def change_save_type(self):
        if self.save_type_picker.currentText() == SaveType.SELECT_PATH.value:
            self.path_select_box.show()
        else:
            self.path_select_box.hide()

    # Select path to save the image
    def select_save_path(self):
        select_save_path = QFileDialog.getExistingDirectory(self.root_widget, 'Open directory')
        self.path_input.setText(select_save_path)

    # Edit task
    def edit_task(self):
        if self.tasks_list.selectedIndexes():
            task_index = self.tasks_list.row(self.tasks_list.selectedItems()[0])
            edit_task_dialog = NewTaskDialog(
                self.list_of_tasks[task_index])
            if edit_task_dialog.exec_() == QDialog.Accepted:
                self.list_of_tasks[task_index] = edit_task_dialog.tasks[edit_task_dialog.task_picker.currentText()].task

                self.generate_list_of_tasks()

            else:
                pass

    # Add new task
    def add_task(self):
        new_task_dialog = NewTaskDialog()
        if new_task_dialog.exec_() == QDialog.Accepted:
            self.list_of_tasks.append(new_task_dialog.tasks[new_task_dialog.task_picker.currentText()].task)

        self.generate_list_of_tasks()

    def generate_list_of_tasks(self):
        self.tasks_list.clear()
        for item in self.list_of_tasks:
            self.tasks_list.addItem(item.name_extended)

    # Remove task
    def remove_task(self):
        for selected_task in self.tasks_list.selectedItems():
            index = self.tasks_list.row(selected_task)
            self.list_of_tasks.remove(self.list_of_tasks[self.tasks_list.row(selected_task)])
            self.generate_list_of_tasks()
            try:
                self.tasks_list.setCurrentRow(index)
            except IndexError:
                pass



from PySide6.QtCore import (QMetaObject, Qt)
from PySide6.QtWidgets import (QDialog, QDialogButtonBox,
                               QSizePolicy, QGridLayout, QVBoxLayout, QLayout, QLabel, QComboBox, QFrame)

from imgmanip.models.sort import Sort


class SortImagesDialog(QDialog):
    def __init__(self, sort_type: Sort):
        super().__init__()
        self.sort_type = sort_type

        if not self.objectName():
            self.setObjectName(u"self")

        self.setWindowTitle('Sort images')

        self.grid_layout = QGridLayout(self)
        self.grid_layout.setObjectName(u"grid_layout")

        # Main layout
        self.vertical_layout = QVBoxLayout()
        self.vertical_layout.setObjectName(u"vertical_layout")
        self.vertical_layout.setSizeConstraint(QLayout.SetDefaultConstraint)

        # Sort label
        self.sort_picker_frame = QFrame()
        self.sort_picker_box = QVBoxLayout()

        self.sort_label = QLabel(self)
        self.sort_label.setObjectName(u"sort_label")
        self.sort_label.setText("Sort")

        # Task input
        self.sort_picker = QComboBox(self)
        self.sort_picker.addItem("unsorted")
        self.sort_picker.addItem("alphabetically")
        self.sort_picker.addItem("types")
        self.sort_picker.setCurrentText(self.sort_type.value)
        self.sort_picker.setObjectName(u"sort_picker")

        self.sort_picker_box.addWidget(self.sort_label)
        self.sort_picker_box.addWidget(self.sort_picker)
        self.sort_picker_frame.setLayout(self.sort_picker_box)

        self.vertical_layout.addWidget(self.sort_picker_frame)
        self.vertical_layout.addStretch(20)

        self.grid_layout.addLayout(self.vertical_layout, 0, 0, 1, 1)

        # Button "Cancel" and "Ok"
        self.button_box = QDialogButtonBox(self)
        self.button_box.setObjectName(u"button_box")
        size_policy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        size_policy3.setHorizontalStretch(0)
        size_policy3.setVerticalStretch(0)
        size_policy3.setHeightForWidth(self.button_box.sizePolicy().hasHeightForWidth())
        self.button_box.setSizePolicy(size_policy3)
        self.button_box.setOrientation(Qt.Horizontal)
        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        self.button_box.button(QDialogButtonBox.Cancel).setStyleSheet(
            'border: 0.5px solid #555555; background-color: #202124; color: #99A2FF')

        self.button_box.button(QDialogButtonBox.Ok).setStyleSheet('background-color: #99A2FF; color: #111111;')
        self.button_box.setCenterButtons(True)

        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)
        self.grid_layout.addWidget(self.button_box, 1, 0, 1, 1)

        QMetaObject.connectSlotsByName(self)



import os
from os.path import basename

from PIL import Image

from imgmanip.functions.get_ext import get_ext_from_file
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import ConvertTask


def convert_image(image, convert_task: ConvertTask, save_type: SaveType, out_path=None):
    img = Image.open(image)

    if save_type == SaveType.SELECT_PATH:
        new_file_name = out_path + '/' + basename(image).replace(get_ext_from_file(image),
                                                                 str(convert_task.convert_ext.value))
    elif save_type == SaveType.IMAGE_PATH:
        new_file_name = image.replace(get_ext_from_file(image), '_new' + str(convert_task.convert_ext.value))
    else:
        os.remove(image)
        new_file_name = image.replace(get_ext_from_file(image), str(convert_task.convert_ext.value))

    img.save(new_file_name)
    return new_file_name



import os
from multiprocessing import cpu_count
from pathlib import Path
from sys import platform

import yaml

if platform == "linux" or platform == "linux2":
    config_path = str(Path.home()) + '/.local/share/imgmanip'
    config_file_path = config_path + '/config.yaml'
elif platform == "win32":
    config_path = str(Path.home()) + '\\AppData\\Local\\imgmanip'
    config_file_path = config_path + '\\config.yaml'
else:
    config_path = None
    config_file_path = None

default_config = {
    'cpu_count': int(cpu_count() / 2),
    'maximized_window': False,
}


def write_config(config):
    os.makedirs(config_path, exist_ok=True)
    config_file = open(config_file_path, 'w')
    yaml.dump(config, config_file)


def read_config():
    config_file = open(config_file_path)
    config_data = yaml.load(config_file.read(), Loader=yaml.FullLoader)
    return config_data


if not os.path.exists(config_file_path):
    write_config(default_config)



from PySide6.QtCore import (QMetaObject,
                            Qt)
from PySide6.QtWidgets import (QDialog, QDialogButtonBox,
                               QSizePolicy, QGridLayout, QVBoxLayout, QLayout, QLabel, QComboBox, QFrame)

from imgmanip.widgets.task_frames.color_detection_task_frame import ColorDetectionTaskFrame
from imgmanip.widgets.task_frames.compress_task_frame import CompressTaskFrame
from imgmanip.widgets.task_frames.convert_task_frame import ConvertTaskFrame
from imgmanip.widgets.task_frames.flip_task_frame import FlipTaskFrame
from imgmanip.widgets.task_frames.invert_task_frame import InvertTaskFrame
from imgmanip.widgets.task_frames.resize_task_frame import ResizeTaskFrame
from imgmanip.widgets.task_frames.convert_color_mode_task_frame import ConvertColorModeTaskFrame


class NewTaskDialog(QDialog):
    def __init__(self, task_to_edit=None):
        super().__init__()
        if not self.objectName():
            self.setObjectName(u"self")

        self.setWindowTitle('New task')
        self.task_to_edit = task_to_edit

        self.grid_layout = QGridLayout(self)
        self.grid_layout.setObjectName(u"grid_layout")

        # Main layout
        self.vertical_layout = QVBoxLayout()
        self.vertical_layout.setObjectName(u"vertical_layout")
        self.vertical_layout.setSizeConstraint(QLayout.SetDefaultConstraint)

        # Task label
        self.task_picker_frame = QFrame()
        self.task_picker_box = QVBoxLayout()
        self.task_picker_box.setContentsMargins(10, 0, 10, 0)

        self.task_label = QLabel(self)
        self.task_label.setObjectName(u"condition_label")
        self.task_label.setText("Task")

        self.tasks = {
            'resize': ResizeTaskFrame(self.task_to_edit),
            'invert': InvertTaskFrame(self.task_to_edit),
            'flip': FlipTaskFrame(self.task_to_edit),
            'convert': ConvertTaskFrame(self.task_to_edit),
            'compress': CompressTaskFrame(self.task_to_edit),
            'color_detection': ColorDetectionTaskFrame(self.task_to_edit),
            'convert_color_mode': ConvertColorModeTaskFrame(self.task_to_edit),
        }

        # Task input
        self.task_picker = QComboBox(self)
        self.task_picker.addItems([task for task in self.tasks])

        self.task_picker.setObjectName(u"combo_box")
        self.task_picker.currentTextChanged.connect(self.change_task)
        self.task_picker_box.addWidget(self.task_label)
        self.task_picker_box.addWidget(self.task_picker)
        self.task_picker_frame.setLayout(self.task_picker_box)

        self.vertical_layout.addWidget(self.task_picker_frame)

        # Widget frames
        for task in self.tasks:
            self.vertical_layout.addWidget(self.tasks[task])

        self.grid_layout.addLayout(self.vertical_layout, 0, 0, 1, 1)

        # Button "Cancel" and "Ok"
        self.button_box = QDialogButtonBox(self)
        self.button_box.setObjectName(u"button_box")
        size_policy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        size_policy3.setHorizontalStretch(0)
        size_policy3.setVerticalStretch(0)
        size_policy3.setHeightForWidth(self.button_box.sizePolicy().hasHeightForWidth())
        self.button_box.setSizePolicy(size_policy3)
        self.button_box.setOrientation(Qt.Horizontal)
        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        self.button_box.button(QDialogButtonBox.Cancel).setStyleSheet(
            'border: 0.5px solid #555555; background-color: #202124; color: #99A2FF')

        self.button_box.button(QDialogButtonBox.Ok).setStyleSheet('background-color: #99A2FF; color: #111111;')
        self.button_box.setCenterButtons(True)

        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)
        self.grid_layout.addWidget(self.button_box, 1, 0, 1, 1)

        if task_to_edit:
            self.edit_dialog()

        self.change_task()

        QMetaObject.connectSlotsByName(self)

    # Change labels and inputs
    def change_task(self):
        # RESIZE TASK
        selected_task = self.task_picker.currentText()
        for task in self.tasks:
            self.tasks[task].hide()

        self.tasks[selected_task].show()
        self.setFixedSize(self.sizeHint())

    # Edit dialog
    def edit_dialog(self):
        self.task_picker.setCurrentText(self.task_to_edit.name)
        self.tasks[self.task_to_edit.name].task = self.task_to_edit

        self.change_task()



import multiprocessing

from PySide6.QtCore import (QMetaObject, Qt)
from PySide6.QtGui import QIntValidator, QFont
from PySide6.QtWidgets import (QDialog, QDialogButtonBox,
                               QSizePolicy, QGridLayout, QVBoxLayout, QLayout, QLabel, QLineEdit, QCheckBox, QComboBox)

from imgmanip.widgets.horizontal_line import HorizontalLine


class PreferencesDialog(QDialog):
    def __init__(self, config):
        super().__init__()
        if not self.objectName():
            self.setObjectName(u"self")

        self.config = config
        self.cpu_count_max = multiprocessing.cpu_count() - 1
        self.setWindowTitle('Preferences')

        self.grid_layout = QGridLayout(self)
        self.grid_layout.setObjectName(u"grid_layout")

        # Main layout
        self.vertical_layout = QVBoxLayout()
        self.vertical_layout.setObjectName(u"vertical_layout")
        self.vertical_layout.setSizeConstraint(QLayout.SetDefaultConstraint)

        int_validator = QIntValidator()
        smaller_font = QFont()
        smaller_font.setPixelSize(12)

        # ===============WINDOW=================
        self.window_part_label = QLabel()
        self.window_part_label.setText('Window settings')

        self.fullscreen_checkbox = QCheckBox()
        self.fullscreen_checkbox.setText('Maximized window')
        self.fullscreen_checkbox.setChecked(self.config['maximized_window'])
        self.fullscreen_checkbox.stateChanged.connect(self.value_changed)

        # ===============TASK SETTINGS=================
        self.task_part_label = QLabel()
        self.task_part_label.setText('Task settings')

        self.cpu_count_label = QLabel()
        self.cpu_count_label.setText('CPU threads')
        self.cpu_max_label = QLabel()
        self.cpu_max_label.setText(f'max: {self.cpu_count_max}')
        self.cpu_max_label.setFont(smaller_font)
        self.cpu_count_input = QLineEdit(self)
        self.cpu_count_input.setObjectName(u"cpu_count")
        self.cpu_count_input.setText(str(self.config['cpu_count']))
        self.cpu_count_input.textChanged.connect(self.value_changed)
        self.cpu_count_input.setValidator(int_validator)

        self.vertical_layout.addWidget(self.window_part_label)
        self.vertical_layout.addWidget(self.fullscreen_checkbox)
        self.vertical_layout.addWidget(HorizontalLine())

        self.vertical_layout.addWidget(self.task_part_label)
        self.vertical_layout.addWidget(self.cpu_count_label)
        self.vertical_layout.addWidget(self.cpu_count_input)
        self.vertical_layout.addWidget(self.cpu_max_label)
        self.vertical_layout.addWidget(HorizontalLine())

        self.button_box = QDialogButtonBox(self)
        self.button_box.setObjectName(u"button_box")
        size_policy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        size_policy3.setHorizontalStretch(0)
        size_policy3.setVerticalStretch(0)
        size_policy3.setHeightForWidth(self.button_box.sizePolicy().hasHeightForWidth())
        self.button_box.setSizePolicy(size_policy3)
        self.button_box.setOrientation(Qt.Horizontal)
        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Save)
        self.button_box.setCenterButtons(True)

        self.grid_layout.addLayout(self.vertical_layout, 0, 0, 1, 1)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        self.grid_layout.addWidget(self.button_box, 1, 0, 1, 1)
        self.setFixedSize(self.sizeHint())

        QMetaObject.connectSlotsByName(self)

    # Update config values
    def value_changed(self):
        # CPU COUNT SETTINGS
        try:
            cpu_count = int(self.cpu_count_input.text().strip())
            if cpu_count > self.cpu_count_max:
                self.cpu_count_input.setText(str(self.config['cpu_count']))
            else:
                self.config['cpu_count'] = int(self.cpu_count_input.text().strip())

        except ValueError:
            pass

        # FULLSCREEN
        self.config['maximized_window'] = self.fullscreen_checkbox.isChecked()



from PySide6.QtWidgets import QFrame, QVBoxLayout, QLayout, QLabel

from imgmanip.models.task import InvertTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class InvertTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == InvertTask:
            self.task = task
        else:
            self.task = InvertTask()

        self.inverse_box = QVBoxLayout()

        self.description_label = QLabel()
        self.description_label.setText(
            "This task inverts the colors of the image.")

        self.inverse_box.setObjectName(u"inverse_box")
        self.inverse_box.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.inverse_box.addWidget(self.description_label)
        self.inverse_box.addWidget(HorizontalLine())
        self.setLayout(self.inverse_box)
        self.hide()



from os.path import basename

from PIL import Image

from imgmanip.functions.get_ext import get_ext_from_file
from imgmanip.functions.save_img import save_image_path
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import CompressTask


def compress_image(image, compress_task: CompressTask, save_type: SaveType, out_path=None):
    img = Image.open(image)
    image_ext = get_ext_from_file(image)

    new_file_name = save_image_path(image_path=image, save_type=save_type, out_path=out_path)

    if image_ext == '.jpg' or image_ext == '.jpeg':
        img.save(new_file_name, 'JPEG', quality=compress_task.quality)
    else:
        img.save(new_file_name, optimize=True, quality=compress_task.quality)
    return new_file_name



from PIL import Image

from imgmanip.functions.save_img import save_image_path
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import ConvertColorModeTask


def convert_color_mode_image(image, convert_color_mode_task: ConvertColorModeTask, save_type: SaveType, out_path=None):
    img = Image.open(image).convert(convert_color_mode_task.color_mode.value)

    new_file_name = save_image_path(image_path=image, save_type=save_type, out_path=out_path)

    img.save(new_file_name)
    return new_file_name



from imgmanip.models.axis import Axis
from imgmanip.models.color_mode import ColorMode
from imgmanip.models.image_extension import ImageExtension
from imgmanip.models.resize_type import ResizeType


class Task:
    def __init__(self):
        pass


class ResizeTask(Task):
    def __init__(self, resize_type: ResizeType, new_width=800, new_height=600, percent=100):
        super().__init__()
        self.name = 'resize'
        self.resize_type = resize_type
        self.new_width = new_width
        self.new_height = new_height
        self.percent = percent
        self.name_extended = f'Resize {new_width}x{new_height}'


class InvertTask(Task):
    def __init__(self):
        super().__init__()
        self.name = 'invert'
        self.name_extended = f'Invert'


class ConvertTask(Task):
    def __init__(self, convert_ext: ImageExtension):
        super().__init__()
        self.name = 'convert'
        self.convert_ext = convert_ext
        self.name_extended = f'Convert {convert_ext.value}'


class CompressTask(Task):
    def __init__(self, quality=90):
        super().__init__()
        self.name = 'compress'
        self.quality = quality
        self.name_extended = f'Compress {quality}'


class ColorDetectionTask(Task):
    def __init__(self, save_mask: bool, color=None):
        super().__init__()
        self.name = 'color_detection'
        self.color = color
        self.save_mask = save_mask
        self.name_extended = f'Color detection {color}'


class FlipTask(Task):
    def __init__(self, axis=Axis.HORIZONTAL):
        super().__init__()
        self.name = 'flip'
        self.axis = axis
        self.name_extended = f'Flip {axis.value}'


class ConvertColorModeTask(Task):
    def __init__(self, color_mode=ColorMode):
        super().__init__()
        self.name = 'convert_color_mode'
        self.color_mode = color_mode
        self.name_extended = f'Convert color mode {color_mode.value}'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['imgmanip',
 'imgmanip.dialogs',
 'imgmanip.functions',
 'imgmanip.functions.tasks',
 'imgmanip.models',
 'imgmanip.widgets',
 'imgmanip.widgets.main',
 'imgmanip.widgets.task_frames']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.2.0,<10.0.0',
 'PySide6>=6.3.2,<7.0.0',
 'PyYAML>=6.0,<7.0',
 'natsort>=8.2.0,<9.0.0',
 'numpy>=1.23.3,<2.0.0',
 'opencv-python>=4.6.0.66,<5.0.0.0',
 'pyperclip>=1.8.2,<2.0.0',
 'pyqtdarktheme>=1.1.0,<2.0.0']

entry_points = \
{'console_scripts': ['imgmanip = imgmanip.main:main']}

setup_kwargs = {
    'name': 'imgmanip',
    'version': '0.0.5',
    'description': 'Image manipulation tool written in python in which you can select a large number of photos and perform a lot of different operations on them.',
    'long_description': "Image manipulation tool in which you can select a large number of photos and\nperform a lot of different operations on them.\n---\n\n## Installation\n\n### Install from PyPI\n\n```shell\npip install imgmanip\n```\n\n#### If you want the program to be displayed in the system applications menu. (Linux only)\n\n```shell\nsudo ln -s $(whereis imgmanip) /usr/bin/imgmanip\ncurl https://raw.githubusercontent.com/hawier-dev/imgmanip/develop/imgmanip.desktop > ./imgmanip.desktop\nsudo mv ./imgmanip.desktop /usr/share/applications/imgmanip.desktop\n```\n\nThe current list of operations\n\n- **Resize** - resizes the images to the given resolution or by a specified percentage.\n- **Compress** - compresses the image. The lower the 'quality',\n  the smaller the file size.\n- **Invert** - inverts the colors of the image.\n- **Flip** - flips the image in horizontal or vertical axis.\n- **Color detection** - marks where the given color appears in the image.\n  Additionally, it can save the **mask** in .png format.\n- **Convert** - converts the image to the other format.\n- **Convert color mode** - This task converts the image color mode to the selected one.",
    'author': 'Mikołaj Badyl',
    'author_email': 'contact@hawier.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



image_extensions = [".jpg", ".jpeg", ".tif", ".bmp", ".png", ".gif"]


def check_image_file(image):
    for image_ext in image_extensions:
        if image.endswith(image_ext):
            return True
    return False



from os.path import basename

from imgmanip.functions.get_ext import get_ext_from_file
from imgmanip.models.save_type import SaveType


def save_image_path(image_path, save_type: SaveType, out_path=None):
    image_ext = get_ext_from_file(image_path)
    if save_type == SaveType.SELECT_PATH:
        new_file_name = out_path + '/' + basename(image_path)
    elif save_type == SaveType.IMAGE_PATH:
        new_file_name = image_path.replace(image_ext, f'_new{image_ext}')
    else:
        new_file_name = image_path

    return new_file_name



from PIL import Image

from imgmanip.functions.save_img import save_image_path
from imgmanip.models.resize_type import ResizeType
from imgmanip.models.save_type import SaveType
from imgmanip.models.task import ResizeTask


def resize_image(image, resize_task: ResizeTask, save_type: SaveType, out_path=None):
    # Image
    img = Image.open(image)
    width, height = img.size

    new_file_name = save_image_path(image_path=image, save_type=save_type, out_path=out_path)

    if resize_task.resize_type == ResizeType.SIZE:
        img = img.resize((int(resize_task.new_width), int(resize_task.new_height)))
    elif resize_task.resize_type == ResizeType.PERCENTAGE:
        img = img.resize((int((resize_task.percent / 100) * width), int((resize_task.percent / 100) * height)))
    # img = img.resize((int(resize_task.new_width), int(resize_task.new_height)))

    img.save(new_file_name)
    return new_file_name



from PySide6.QtGui import QIntValidator, QDoubleValidator
from PySide6.QtWidgets import QFrame, QVBoxLayout, QLabel, QSizePolicy, QHBoxLayout, QLineEdit, QLayout, QComboBox

from imgmanip.models.resize_type import ResizeType
from imgmanip.models.task import ResizeTask
from imgmanip.widgets.horizontal_line import HorizontalLine


class ResizeTaskFrame(QFrame):
    def __init__(self, task=None):
        super().__init__()

        if task and type(task) == ResizeTask:
            self.task = task
        else:
            self.task = ResizeTask(resize_type=ResizeType.SIZE, new_width=800, new_height=600)

        self.resize_box = QVBoxLayout()
        self.resize_box.setObjectName(u"resize_box")
        self.resize_box.setSizeConstraint(QLayout.SetDefaultConstraint)

        self.description_label = QLabel()
        self.description_label.setText(
            "This task resizes the images to the given resolution or by percentage.")

        # Resize types
        self.resize_type_picker = QComboBox()
        for resize_type in ResizeType:
            self.resize_type_picker.addItem(resize_type.value)

        self.resize_type_picker.setCurrentText(self.task.resize_type.value)
        self.resize_type_picker.currentTextChanged.connect(self.change_type)

        # Name label
        self.resize_label = QLabel(self)
        self.resize_label.setObjectName(u"name_label")
        size_policy_resize_label = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        size_policy_resize_label.setHorizontalStretch(0)
        size_policy_resize_label.setVerticalStretch(0)
        size_policy_resize_label.setHeightForWidth(self.resize_label.sizePolicy().hasHeightForWidth())
        self.resize_label.setSizePolicy(size_policy_resize_label)
        self.resize_label.setText("New size")

        # Resize size type
        # validator
        int_validator = QIntValidator()

        self.resize_size_frame = QFrame()
        self.resize_size_frame.setContentsMargins(0, 0, 0, 0)
        self.resize_inputs_h_box = QHBoxLayout()
        self.resize_inputs_h_box.setContentsMargins(0, 0, 0, 0)
        self.resize_width_input = QLineEdit(self)
        self.resize_width_input.setObjectName(u"resize_width_input")
        self.resize_width_input.setText(str(self.task.new_width))
        self.resize_width_input.setValidator(int_validator)
        self.resize_width_input.textEdited.connect(self.change_resize)

        # X label between width and height -> width x height
        self.x_label = QLabel(self)
        self.x_label.setObjectName(u"name_label")
        self.x_label.setText('x')

        self.resize_height_input = QLineEdit(self)
        self.resize_height_input.setObjectName(u"resize_height_input")
        self.resize_height_input.setText(str(self.task.new_height))
        self.resize_height_input.setValidator(int_validator)
        self.resize_height_input.textEdited.connect(self.change_resize)

        self.resize_inputs_h_box.addWidget(self.resize_width_input)
        self.resize_inputs_h_box.addWidget(self.x_label)
        self.resize_inputs_h_box.addWidget(self.resize_height_input)
        self.resize_size_frame.setLayout(self.resize_inputs_h_box)
        # Resize percentage type
        self.resize_percentage_frame = QFrame()
        self.resize_percentage_frame.setContentsMargins(0, 0, 0, 0)
        self.resize_percentage_h_box = QHBoxLayout()
        self.resize_percentage_h_box.setContentsMargins(0, 0, 0, 0)
        self.percentage_input = QLineEdit(self)
        self.percentage_input.setObjectName(u"percentage_input")
        self.percentage_input.setText(str(self.task.percent).replace('.', ','))
        self.percentage_input.textEdited.connect(self.change_resize)
        # % char
        self.percent_label = QLabel(self)
        self.percent_label.setObjectName(u"name_label")
        self.percent_label.setText('%')

        self.resize_percentage_h_box.addWidget(self.percentage_input)
        self.resize_percentage_h_box.addWidget(self.percent_label)
        self.resize_percentage_frame.setLayout(self.resize_percentage_h_box)

        self.resize_percentage_frame.hide()

        self.resize_box.addWidget(self.description_label)
        self.resize_box.addWidget(HorizontalLine())

        self.resize_box.addWidget(self.resize_label)
        self.resize_box.addWidget(self.resize_type_picker)
        self.resize_box.addWidget(self.resize_size_frame)
        self.resize_box.addWidget(self.resize_percentage_frame)

        self.setLayout(self.resize_box)
        self.change_type()
        self.hide()

    # Change text event for
    def change_resize(self):
        self.task.new_width = int(self.resize_width_input.text())
        self.task.new_height = int(self.resize_height_input.text())
        try:
            self.task.percent = float(self.percentage_input.text())
        except ValueError:
            if str(self.task.percent).endswith('.0'):
                self.percentage_input.setText(str(self.task.percent).replace('.0', ''))
            else:
                self.percentage_input.setText(str(self.task.percent))
        if self.task.resize_type == ResizeType.SIZE:
            self.task.name_extended = f'Resize {self.task.new_width}x{self.task.new_height}'
        else:
            self.task.name_extended = f'Resize {self.task.percent}%'

    # Change type of resize
    def change_type(self):
        current_type = self.resize_type_picker.currentText()
        if current_type == 'Size':
            self.resize_size_frame.show()
            self.resize_percentage_frame.hide()
        elif current_type == 'Percentage':
            self.resize_size_frame.hide()
            self.resize_percentage_frame.show()

        self.task.resize_type = ResizeType(current_type)
        if self.task.resize_type == ResizeType.SIZE:
            self.task.name_extended = f'Resize {self.task.new_width}x{self.task.new_height}'
        else:
            self.task.name_extended = f'Resize {self.task.percent}%'
